@@ -4,7 +4,8 @@ import json
 
 
 class ValidationError(Exception):
-    """It will be raised when validation will be failed."""
+
+    """Validation failed."""
 
     def __init__(self, errors, attr=None):
         self.attr = attr
@@ -14,9 +15,11 @@ class ValidationError(Exception):
             self.errors = [errors]
 
     def to_dict(self):
-        """Return a dictionary representation of the error, with the following keys:
-        - attr: Attribute which contains the error, or "<root>" if it refers to the schema root.
-        - errors: A list of dictionary representations of the errors.
+        """Return a dictionary representation of the error.
+
+        :return: A dict with the keys:
+            - attr: Attribute which contains the error, or "<root>" if it refers to the schema root.
+            - errors: A list of dictionary representations of the errors.
         """
         def exception_to_dict(e):
             try:
