@@ -1,9 +1,10 @@
 """Halogen schema primitives."""
 
 import sys
+import inspect
 
-from . import types
-from . import exceptions
+from halogen import types
+from halogen import exceptions
 
 PY2 = sys.version_info[0] == 2
 
@@ -16,56 +17,75 @@ else:
 BYPASS = lambda value: value
 
 
+def _get_context(func, kwargs):
+    """Prepare a context for the serialization.
+
+    :param func: Function which needs or does not need kwargs.
+    :param kwargs: Dict with context
+    :return: Keywords arguments that function can accept.
+    """
+    argspec = inspect.getargspec(func)
+    if argspec.keywords is not None:
+        return kwargs
+    return dict((arg, kwargs[arg]) for arg in argspec.args if arg in kwargs)
+
+
 class Accessor(object):
 
-    """Object that incapsulates the getter and the setter of the attribute."""
+    """Object that encapsulates the getter and the setter of the attribute."""
 
     def __init__(self, getter=None, setter=None):
+        """Initialize an Accessor object."""
         self.getter = getter
         self.setter = setter
 
-    def get(self, value):
+    def get(self, obj, **kwargs):
         """Get an attribute from a value.
 
-        :param value: Object to get the attribute value from.
+        :param obj: Object to get the attribute value from.
         :return: Value of object's attribute.
         """
         assert self.getter is not None, "Getter accessor is not specified."
         if callable(self.getter):
-            return self.getter(value)
+            return self.getter(obj, **_get_context(self.getter, kwargs))
 
         assert isinstance(self.getter, string_types), "Accessor must be a function or a dot-separated string."
 
         for attr in self.getter.split("."):
-            if isinstance(value, dict):
-                value = value[attr]
+            if isinstance(obj, dict):
+                obj = obj[attr]
             else:
-                value = getattr(value, attr)
-        return value
+                obj = getattr(obj, attr)
 
-    def set(self, result, value):
-        """Set value for result's attribute.
+        if callable(obj):
+            return obj()
 
-        :param result: Result object or dict to assign the attribute to.
+        return obj
+
+    def set(self, obj, value):
+        """Set value for obj's attribute.
+
+        :param obj: Result object or dict to assign the attribute to.
         :param value: Value to be assigned.
         """
         assert self.setter is not None, "Setter accessor is not specified."
         if callable(self.setter):
-            return self.setter(value)
+            return self.setter(obj, value)
 
         assert isinstance(self.setter, string_types), "Accessor must be a function or a dot-separated string."
 
-        def setdefault(result, attr, value):
-            if isinstance(result, dict):
-                result.setdefault(attr, value)
+        def _set(obj, attr, value):
+            if isinstance(obj, dict):
+                obj[attr] = value
             else:
-                setattr(result, attr, value)
+                setattr(obj, attr, value)
             return value
 
         path = self.setter.split(".")
         for attr in path[:-1]:
-            result = setdefault(result, attr, {})
-        setdefault(result, path[-1], value)
+            obj = _set(obj, attr, {})
+
+        _set(obj, path[-1], value)
 
     def __repr__(self):
         """Accessor representation."""
@@ -80,17 +100,20 @@ class Attr(object):
 
     """Schema attribute."""
 
-    def __init__(self, attr_type=None, attr=None, required=True):
+    def __init__(self, attr_type=None, attr=None, required=True, **kwargs):
         """Attribute constructor.
 
         :param attr_type: Type, Schema or constant that does the type conversion of the attribute.
         :param attr: Attribute name, dot-separated attribute path or an `Accessor` instance.
         :param required: Is attribute required to be present.
         """
-        self.attr_type = attr_type or types.Type
+        self.attr_type = attr_type or types.Type()
         self.attr = attr
         self.required = required
 
+        if "default" in kwargs:
+            self.default = kwargs["default"]
+
     @property
     def compartment(self):
         """The key of the compartment this attribute will be placed into (for example: _links or _embedded)."""
@@ -107,14 +130,16 @@ class Attr(object):
 
         :return: `Accessor` instance.
         """
+        if isinstance(self.attr, Accessor):
+            return self.attr
+
+        if callable(self.attr):
+            return Accessor(getter=self.attr)
+
         attr = self.attr or self.name
-
-        if isinstance(attr, Accessor):
-            return attr
-
         return Accessor(getter=attr, setter=attr)
 
-    def serialize(self, value):
+    def serialize(self, value, **kwargs):
         """Serialize the attribute of the input data.
 
         Gets the attribute value with accessor and converts it using the
@@ -126,7 +151,15 @@ class Attr(object):
         :return: Serialized attribute value.
         """
         if types.Type.is_type(self.attr_type):
-            return self.attr_type.serialize(self.accessor.get(value))
+            try:
+                value = self.accessor.get(value, **kwargs)
+            except (AttributeError, KeyError):
+                if not hasattr(self, "default") and self.required:
+                    raise
+                value = self.default() if callable(self.default) else self.default
+
+            return self.attr_type.serialize(value, **_get_context(self.attr_type.serialize, kwargs))
+
         return self.attr_type
 
     def deserialize(self, value):
@@ -134,7 +167,7 @@ class Attr(object):
 
         Get the value from the HAL structure from the attribute's compartment
         using the attribute's name as a key, convert it using the attribute's
-        type. Schema will either return it to the parent schema or will assign
+        type. Schema will either return it to  parent schema or will assign
         to the output value if specified using the attribute's accessor setter.
 
         :param value: HAL structure to get the value from.
@@ -142,12 +175,19 @@ class Attr(object):
         :raises: ValidationError.
         """
         compartment = value
+
         if self.compartment is not None:
             compartment = value[self.compartment]
-        if self.name in compartment:
-            return self.attr_type.deserialize(compartment[self.key])
-        elif self.required:
-            raise exceptions.ValidationError("Missing attribute.", self.key)
+
+        try:
+            value = self.accessor.get(compartment)
+        except KeyError:
+            if hasattr(self, "default"):
+                value = self.default
+            else:
+                raise
+
+        return self.attr_type.deserialize(value)
 
     def __repr__(self):
         """Attribute representation."""
@@ -161,15 +201,16 @@ class Link(Attr):
 
     """Link attribute of a schema."""
 
-    def __init__(self, attr_type=None, attr=None, required=True, curie=None, is_templated=None, media_type=None):
+    def __init__(self, attr_type=None, attr=None, key=None, required=True, curie=None, templated=None, type=None):
         """Link constructor.
 
         :param attr_type: Type, Schema or constant that does the type conversion of the attribute.
         :param attr: Attribute name, dot-separated attribute path or an `Accessor` instance.
+        :param key: Key of the link in the _links compartment, defaults to name.
         :param required: Is this link required to be present.
         :param curie: Link namespace prefix (e.g. "<prefix>:<name>") or Curie object.
-        :param is_templated: Is this link templated.
-        :param media_type: Its value is a string used as a hint to indicate the media type expected when dereferencing
+        :param templated: Is this link templated.
+        :param type: Its value is a string used as a hint to indicate the media type expected when dereferencing
                            the target resource.
         """
         if not types.Type.is_type(attr_type):
@@ -177,34 +218,44 @@ class Link(Attr):
             if attr_type is not None:
                 attr = BYPASS
 
+            attrs = {
+                'templated': templated,
+                'type': type,
+            }
+
             class LinkSchema(Schema):
                 href = Attr(attr_type=attr_type, attr=BYPASS)
 
-                if is_templated is not None:
-                    templated = Attr(attr=lambda value: is_templated)
+                if attrs['templated'] is not None:
+                    templated = Attr(attr=lambda value: templated)
 
-                if media_type is not None:
-                    type = Attr(attr=lambda value: media_type)
+                if attrs['type'] is not None:
+                    type = Attr(attr=lambda value: type)
 
             attr_type = LinkSchema
 
         super(Link, self).__init__(attr_type=attr_type, attr=attr, required=required)
         self.curie = curie
+        self._key = key
 
     @property
     def compartment(self):
-        """Links are placed in the _links."""
+        """Return the compartment in which Links are placed (_links)."""
         return "_links"
 
     @property
     def key(self):
-        """Links support curies."""
+        """The key of the this attribute will be placed into (within it's compartment).
+
+        :note: Links support curies.
+        """
+        name = self._key or self.name
         if self.curie is None:
-            return self.name
-        return ":".join((self.curie.name, self.name))
+            return name
+        return ":".join((self.curie.name, name))
 
     def deserialize(self, value):
-        """Links don't support deserialization."""
+        """Link doesn't support deserialization."""
         raise NotImplementedError
 
 
@@ -228,22 +279,22 @@ class Curie(object):
 
     """Curie object."""
 
-    def __init__(self, name, href, is_templated=None, media_type=None):
+    def __init__(self, name, href, templated=None, type=None):
         """Curie constructor.
 
         :param href: Curie link href value.
-        :param is_templated: Is this curie link templated.
-        :param media_type: Its value is a string used as a hint to indicate the media type expected when dereferencing
+        :param templated: Is this curie link templated.
+        :param type: Its value is a string used as a hint to indicate the media type expected when dereferencing
                            the target resource.
         """
         self.name = name
         self.href = href
 
-        if is_templated is not None:
-            self.templated = is_templated
+        if templated is not None:
+            self.templated = templated
 
-        if media_type is not None:
-            self.type = media_type
+        if type is not None:
+            self.type = type
 
 
 class Embedded(Attr):
@@ -251,6 +302,12 @@ class Embedded(Attr):
     """Embedded attribute of schema."""
 
     def __init__(self, attr_type=None, attr=None, curie=None):
+        """Embedded constructor.
+
+        :param attr_type: Type, Schema or constant that does the type conversion of the attribute.
+        :param attr: Attribute name, dot-separated attribute path or an `Accessor` instance.
+        :param curie: The curie used for this embedded attribute.
+        """
         super(Embedded, self).__init__(attr_type, attr)
         self.curie = curie
 
@@ -266,9 +323,6 @@ class Embedded(Attr):
             return self.name
         return ":".join((self.curie.name, self.name))
 
-    def serialize(self, value):
-        return super(Embedded, self).serialize(value)
-
 
 class _Schema(types.Type):
 
@@ -287,14 +341,14 @@ class _Schema(types.Type):
         return schema
 
     @classmethod
-    def serialize(cls, value):
+    def serialize(cls, value, **kwargs):
         result = {}
         for attr in cls.__attrs__:
             compartment = result
             if attr.compartment is not None:
                 compartment = result.setdefault(attr.compartment, {})
             try:
-                compartment[attr.key] = attr.serialize(value)
+                compartment[attr.key] = attr.serialize(value, **kwargs)
             except (AttributeError, KeyError):
                 if attr.required:
                     raise
@@ -322,6 +376,11 @@ class _Schema(types.Type):
             except exceptions.ValidationError as e:
                 e.attr = attr.name
                 errors.append(e)
+            except KeyError:
+                if attr.required:
+                    e = exceptions.ValidationError("Missing attribute.", attr.name)
+                    e.attr = attr.name
+                    errors.append(e)
 
         if errors:
             raise exceptions.ValidationError(errors)
@@ -329,11 +388,16 @@ class _Schema(types.Type):
         if output is None:
             return result
         for attr in cls.__attrs__:
-            attr.accessor.set(output, result[attr.name])
+            if attr.name in result:
+                attr.accessor.set(output, result[attr.name])
 
 
 class _SchemaType(type):
+
+    """A type used to create Schemas."""
+
     def __init__(cls, name, bases, clsattrs):
+        """Create a new _SchemaType."""
         cls.__class_attrs__ = []
         curies = set([])
 
@@ -346,7 +410,7 @@ class _SchemaType(type):
                 if not hasattr(value, "name"):
                     value.name = name
 
-                if isinstance(value, Link):
+                if isinstance(value, (Link, Embedded)):
                     curie = getattr(value, "curie", None)
                     if curie is not None:
                         curies.add(curie)
@@ -359,6 +423,7 @@ class _SchemaType(type):
                     href=Attr(),
                     name=Attr(),
                     templated=Attr(required=False),
+                    type=Attr(required=False),
                 ),
                 attr=lambda value: list(curies),
                 required=False,
@@ -373,3 +438,4 @@ class _SchemaType(type):
 
 
 Schema = _SchemaType("Schema", (_Schema, ), {"__doc__": _Schema.__doc__})
+"""Schema is the basic class used for setting up schemas."""
